local CipherModule = {}

-- مولّد البذرة البسيط
local function simpleSeed(key)
  local seed = 0
  for i = 1, #key do
    seed = (seed * 31 + key:byte(i)) % 1000000007
  end
  return seed
end

-- المولّد الخطي المتكرر (LCG)
local function lcg(seed)
  local m, a, c = 2^31 - 1, 1103515245, 12345
  local state = seed
  return function()
    state = (a * state + c) % m
    return state / m
  end
end

-- نبني الخرائط بناءً على مجموعة ثابتة من جميع بايتات 0–255
local function build_maps(key, salt)
  local chars = {}
  for b = 0, 255 do
    chars[#chars + 1] = string.char(b)
  end

  local seed = simpleSeed(key .. salt)
  local rnd  = lcg(seed)
  local codes, used = {}, {}

  for i = 1, #chars do
    local code
    repeat
      local parts = {}
      for j = 1, 4 do
        parts[j] = string.char(math.floor(rnd() * 26) + 65)
      end
      code = table.concat(parts)
    until not used[code]
    used[code] = true
    codes[i] = code
  end

  local map, reverseMap = {}, {}
  for i, ch in ipairs(chars) do
    map[ch] = codes[i]
    reverseMap[codes[i]] = ch
  end

  return map, reverseMap
end

-- مولّد الـ salt عشوائي (16 حرفاً إنجليزياً كبيراً)
local function generateSalt()
  local salt = {}
  for i = 1, 16 do
    salt[i] = string.char(math.random(65, 90))
  end
  return table.concat(salt)
end

-- الدالة الرئيسية للتشفير/فكّ التشفير
function CipherModule.cipher(text, key)
  assert(key and #key > 0, "Encryption key is required.")
  local isDecrypt = key:sub(1, 7) == "decrypt"
  local realKey   = isDecrypt and key:sub(8) or key

  local prefix = [[
! [.2.0.note]. 
t=t,&z=gue\n=7474#84=\n. "
]]

  local salt, payload

  if isDecrypt then
    if text:sub(1, #prefix) ~= prefix then
      error("Invalid prefix in cipher text.")
    end
    text    = text:sub(#prefix + 1)
    salt    = text:sub(1, 16)
    payload = text:sub(17)
  else
    salt    = generateSalt()
    payload = text
  end

  local map, reverseMap = build_maps(realKey, salt)

  if isDecrypt then
    local result, i = {}, 1
    while i <= #payload do
      local chunk = payload:sub(i, i + 3)
      local decoded = reverseMap[chunk]
      if not decoded then
        error("Invalid cipher text or wrong key/salt at position " .. i)
      end
      table.insert(result, decoded)
      i = i + 4
    end
    return table.concat(result)
  else
    local result = {}
    for i = 1, #payload do
      local ch   = payload:sub(i, i)
      local code = map[ch]
      if not code then
        error("No mapping for character (byte) at position " .. i)
      end
      table.insert(result, code)
    end
    return prefix .. salt .. table.concat(result)
  end
end

return CipherModule